from django.core.management.base import BaseCommand
from django.core.cache import cache
from webp_converter.utils import delete_webp_folder
from webp_converter.models import WebPImage
from webp_converter.utils import make_image_key, webp_image_querysets


class Command(BaseCommand):
    help = 'Clear webp cached images'

    def handle(self, *args, **options):
        total_count = WebPImage.objects.all().count()
        for qs in webp_image_querysets(total_count):
            for webp_image in qs:
                key = make_image_key(webp_image.static_path, webp_image.quality)
                cache.delete(key)
        WebPImage.objects.all().delete()
        delete_webp_folder()
        self.stdout.write('Successfully cleared cache')



from django.test import TestCase
from django.contrib.staticfiles.templatetags.staticfiles import static
from webp_converter.templatetags.webp_converter import static_webp
from webp_converter.models import WebPImage


class TestTemplateTags(TestCase):

    def setUp(self):
        self.static_path = 'django-test-image.png'
        self.webp_image = WebPImage.objects.create(static_path=self.static_path)

    def test_static_webp_templatetag(self):
        context = {'webp_compatible': True}
        result = static_webp(context=context, static_path=self.static_path)
        assert result == self.webp_image.url

    def test_missing_context(self):
        with self.assertRaises(Exception):
            static_webp(context={}, static_path=self.static_path)

    def test_no_webp_support(self):
        context = {'webp_compatible': False}
        result = static_webp(context=context, static_path=self.static_path)
        assert result == static(self.webp_image.static_path)




from django.test import TestCase
from django.core.cache import cache
from django.core.management import call_command
from webp_converter.models import WebPImage
from webp_converter.utils import make_image_key


class TestManagementCommands(TestCase):

    def test_clear_cache(self):
        static_path = 'django-test-image.png'
        quality = 80
        key = make_image_key(static_path, quality)
        cache.set(key, 'test')
        assert cache.get(key) == 'test'
        WebPImage.objects.create(
            static_path=static_path, quality=quality)
        assert WebPImage.objects.all().count() == 1
        call_command('clear_webp_cache')
        assert cache.get(key) is None
        assert WebPImage.objects.all().count() == 0




from django import template
from django.contrib.staticfiles.templatetags.staticfiles import static
from webp_converter.utils import make_image_key
from django.core.cache import cache
from webp_converter.models import WebPImage

register = template.Library()


@register.simple_tag(takes_context=True)
def static_webp(context, static_path, quality=None):
    try:
        webp_compatible = context['webp_compatible']
    except KeyError:
        raise IOError("'webp_converter.context_processors.webp_support' "
                      "needs to be added to your context processors.")
    if not webp_compatible:
        return static(static_path)
    key = make_image_key(static_path, quality)
    webp_image_url = cache.get(key)
    if not webp_image_url:
        webp_image, _ = WebPImage.objects.get_or_create(
            static_path=static_path, quality=quality)
        webp_image.save_image()
        webp_image_url = webp_image.url
        cache.set(key, webp_image_url)
    return webp_image_url



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='WebPImage',
            fields=[
                ('id', models.AutoField(serialize=False, verbose_name='ID', auto_created=True, primary_key=True)),
                ('static_path', models.CharField(max_length=512)),
                ('quality', models.PositiveIntegerField(blank=True, null=True)),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='webpimage',
            unique_together=set([('static_path', 'quality')]),
        ),
    ]



from django.test import TestCase
from django.http import HttpRequest
from webp_converter.context_processors import webp_support


class TestContextProcessors(TestCase):

    def test_webp_support_true(self):
        request = HttpRequest()
        request.META['HTTP_ACCEPT'] =\
            'text/html,application/xhtml+xml,application/xml;' \
            'q=0.9,image/webp,*/*;q=0.8'
        assert webp_support(request) == {'webp_compatible': True}

    def test_webp_support_false(self):
        request = HttpRequest()
        request.META['HTTP_ACCEPT'] =\
            'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8'
        assert webp_support(request) == {'webp_compatible': False}



import hashlib
import shutil
import os
from django.core.files.storage import default_storage
from django.utils.encoding import force_bytes
from webp_converter.conf.settings import WEBP_CONVERTER_PREFIX
from webp_converter.models import WebPImage


def make_image_key(*args):
    md5_hash = hashlib.md5(force_bytes(':'.join(str(arg) for arg in args)))
    return "webp_converter:%s" % md5_hash.hexdigest()


def webp_image_querysets(total_count, object_number=100):
    for index in range(0, total_count, object_number):
        yield WebPImage.objects.all()[index: index + object_number]


def delete_webp_folder():
    webp_path = default_storage.path(WEBP_CONVERTER_PREFIX)
    if os.path.exists(webp_path):
        shutil.rmtree(default_storage.path(WEBP_CONVERTER_PREFIX))



import mock
import shutil
from datetime import date
import os
from django.test import TestCase
from django.core.files.storage import default_storage
from django.contrib.staticfiles import finders
from django.conf import settings
from webp_converter.models import WebPImage
from webp_converter.conf.settings import WEBP_CONVERTER_PREFIX
from webp_converter.utils import delete_webp_folder


class TestModels(TestCase):
    def setUp(self):
        self.webp_image = WebPImage(
            static_path='django-test-image.png', quality=80)
        self.invalid_webp_image = WebPImage(
            static_path='missing.png', quality=80)

    def test_invalid_image_absolute_path(self):
        with self.assertRaisesMessage(Exception, "Can't find static image."):
            return self.invalid_webp_image.image_absolute_path

    def test_valid_image_absolute_path(self):
        assert self.webp_image.image_absolute_path == finders.find(
            'django-test-image.png')

    def test_webp_relative_path(self):
        assert self.webp_image.webp_relative_path ==\
               '{prefix}/98/6ff1a1/django-test-image.webp'.format(
                   prefix=WEBP_CONVERTER_PREFIX)

    def test_url(self):
        assert self.webp_image.url ==\
            settings.MEDIA_URL + self.webp_image.webp_relative_path

    def test_save_image_kwargs(self):
        save_image_kwargs = self.webp_image._get_save_image_kwargs()
        assert save_image_kwargs == {
            'format': 'WEBP',
            'quality': 80,
            'fp': self.webp_image.webp_absolute_path}

    def test_save_new_image(self):
        delete_webp_folder()
        self.webp_image.save_image()
        assert os.path.exists(self.webp_image.webp_absolute_path)




import os
from django.core.files.storage import default_storage
from django.test import TestCase
from webp_converter import utils
from webp_converter.conf import settings as webp_settings
from webp_converter.models import WebPImage


class TestUtils(TestCase):
    def test_make_image_key(self):
        key = utils.make_image_key('static/path/image.jpg', 60)
        assert key == webp_settings.WEBP_CONVERTER_CACHE_PREFIX +\
            ':3eac052ec0eaf68475576e75e2305f2b'

    def test_webp_querysets(self):
        WebPImage.objects.create(static_path='image_1.png', quality=80)
        WebPImage.objects.create(static_path='image_2.png', quality=80)
        WebPImage.objects.create(static_path='image_3.png', quality=80)
        ids = [
            item.id
            for qs in utils.webp_image_querysets(3, 2)
            for item in qs]
        assert ids == [1, 2, 3]

    def test_delete_webp_folder(self):
        webp_image = WebPImage.objects.create(
            static_path='django-test-image.png', quality=60)
        webp_image.save_image()
        webp_path = default_storage.path(webp_settings.WEBP_CONVERTER_PREFIX)
        self.assertTrue(os.path.exists(webp_path))
        utils.delete_webp_folder()
        self.assertFalse(os.path.exists(webp_path))



import os
import hashlib
from PIL import Image
from django.core.files.storage import default_storage
from django.contrib.staticfiles import finders
from django.db import models
from django.utils.encoding import force_bytes
from django.conf import settings
from webp_converter.conf.settings import WEBP_CONVERTER_PREFIX


class WebPImage(models.Model):
    static_path = models.CharField(max_length=512)
    quality = models.PositiveIntegerField(blank=True, null=True)

    class Meta:
        unique_together = (('static_path', 'quality'),)

    @property
    def image_absolute_path(self):
        """
        The full image file path found from the static path
        """
        full_image_path = finders.find(self.static_path)
        if not full_image_path:
            raise Exception("Can't find static image.")
        return full_image_path

    @property
    def webp_relative_path(self):
        key = hashlib.md5(
            force_bytes(self.static_path + str(self.quality))
        ).hexdigest()
        return '{prefix}/{key_1}/{key_2}/{path}.webp'.format(
            prefix=WEBP_CONVERTER_PREFIX,
            key_1=key[:2],
            key_2=key[2:8],
            path=''.join(self.static_path.split('.')[:-1]))

    @property
    def webp_absolute_path(self):
        """
        The full image file path of the webp image
        """
        return default_storage.path(self.webp_relative_path)

    @property
    def url(self):
        return default_storage.url(self.webp_relative_path)

    def save_image(self):
        image = Image.open(self.image_absolute_path)
        # Create directory structure if it does not exist
        webp_dirname = os.path.dirname(self.webp_absolute_path)
        if not os.path.exists(webp_dirname):
            os.makedirs(webp_dirname)
        image.save(**self._get_save_image_kwargs())

    def _get_save_image_kwargs(self):
        kwargs = {
            'fp': self.webp_absolute_path,
            'format': 'WEBP'}
        if self.quality:
            kwargs['quality'] = self.quality
        return kwargs



import os
from setuptools import setup, find_packages

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-webp-converter',
    version='0.1',
    packages=find_packages(exclude=('django_webp_converter',)),
    include_package_data=True,
    license='BSD License',
    description='A simple Django app to conduct Web-based polls.',
    long_description=README,
    author='Tom Miller',
    author_email='02millert@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',  # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        # Replace these appropriately if you are stuck on Python 2.
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=[
        'django',
        'pillow',
    ],
)
